var indexController = {

	intervalID : null,
	selectedNZB : null,
	displayPanel : null,
	model : null,
	initialUpdate : false,
	
	/***/
	/*** Login and logout ***/
	/***/
	startApplication : function() {
	
		var apiKey = "<unknown>";
		try {
			apiKey = $.jStorage.get("apikey");
		} 
		catch (err) {
		}
		
		if (apiKey) {
			ServerHandler.apikey = apiKey;
		}
		ServerHandler.serverURL = "/api";
		ServerHandler.authenticationFailedHandler = function() {
			ServerHandler.authenticationFailedHandler = null;
			indexController.login();
		};
		
		//disable file upload on iOS
		if (navigator.userAgent.match(/(iPad|iPhone|iPod)/i)) {
			document.getElementById("upload1").disabled = true;
			document.getElementById("btnUploadSubmit").disabled = true;
		}
		
		//no-touch class for no touch specific CSS rules
		if (!("ontouchstart" in document.documentElement)) {
			document.documentElement.className += " no-touch";
		}
		
		indexController.login();
	},

	login : function() {
		ServerHandler.login(function(loginResult) {
			if (!loginResult) {
				indexController.clearUI();
				indexController.showPanel("loginPanel");
				if (indexController.intervalID) {
					clearInterval(indexController.intervalID);
					indexController.intervalID = null;
				}
			} else {
				indexController.initialUpdate = true;
				indexController.hideAllPanels();
				if (!indexController.intervalID) {
					indexController.intervalID = setInterval(indexController.tmrUIUpdateTick, 3000);
				}
				ServerHandler.authenticationFailedHandler = function() {
					ServerHandler.authenticationFailedHandler = null;
					indexController.login();
				};
				indexController.tmrUIUpdateTick();
			}
		});
	},
	
	logout : function() {
		if (indexController.intervalID) {
			clearInterval(indexController.intervalID);
			indexController.intervalID = null;
		}
		
		try {
			$.jStorage.deleteKey('apikey');
		} 
		catch (err) {
		}

		indexController.clearUI();
		indexController.showPanel("loginPanel");
	},
	
	
	
	
	/***/
	/*** UI helper functions ***/
	/***/
	
	tmrPanelSlideIn : function() {
		var currentTop = parseInt(document.getElementById(indexController.displayPanel).style.top);
		var divHeight = document.getElementById(indexController.displayPanel).offsetHeight;
		var moveStep = (currentTop/4);
		
		document.getElementById(indexController.displayPanel).style.top = currentTop-moveStep+"px";
		
		if (parseInt(document.getElementById(indexController.displayPanel).style.top)<0) {
			setTimeout(indexController.tmrPanelSlideIn, 10);
		}
	},
	
	tmrPanelSlideOut : function() {
	
		var currentTop = parseInt(document.getElementById(indexController.displayPanel).style.top);
		var divHeight = document.getElementById(indexController.displayPanel).offsetHeight;
		var moveStep = -5+(currentTop/4);
		
		document.getElementById(indexController.displayPanel).style.top = currentTop+moveStep+"px";
		
		if (parseInt(document.getElementById(indexController.displayPanel).style.top)>-divHeight) {
			setTimeout(indexController.tmrPanelSlideOut, 10);
		} else {
			indexController.displayPanel = null;
			document.getElementById("blackOverlay").style.display = "none";
			document.getElementById("addNZBPanel").style.display = "none";
			document.getElementById("loginPanel").style.display = "none";
			document.getElementById("removeNZBPanel").style.display = "none";
			document.getElementById("removeAllNZBPanel").style.display = "none";
		}
	},
			
	showPanel : function(panel) {
		document.getElementById("upload1").value = "";
		document.getElementById("blackOverlay").style.display = "block";
		indexController.displayPanel = panel;
		document.getElementById(panel).style.display = "block";
		document.getElementById(panel).style.top = -document.getElementById(panel).offsetHeight+"px";
		setTimeout(indexController.tmrPanelSlideIn, 30);
	},
	
	hideAllPanels : function() {
		if (indexController.displayPanel) {
			setTimeout(indexController.tmrPanelSlideOut, 30);
		}
	},
		
			
	/***/
	/*** UI Functions ***/
	/***/
	
	setSelectedNZB : function(id, selectedRow) {
	
		//do magic here
		indexController.selectedNZB = null;
		
		if (indexController.model.nzbs) {
			for (i=0;i<indexController.model.nzbs.length;i++) {
				if (indexController.model.nzbs[i].id==id) {
					indexController.selectedNZB = indexController.model.nzbs[i];
				}
			}
		}
		
		var childNoted = document.getElementById('nzbList').childNodes[0].childNodes;
		for (i=0; i<childNoted.length; i++) {
			var node = childNoted[i];
			var isSelected = selectedRow==node;
			node.setAttribute("class", "nzbListItem"+(isSelected?" selected":""));
		}
		
		indexController.tmrUIUpdateTick();
	},
	
	clearSelection : function() {
		indexController.selectedNZB = null;
		indexController.displayPanel = null;
		indexController.model = null;
		
		indexController.tmrUIUpdateTick();
	},


	updateModel : function(callback) {
		
		var url = "/app/webUpdate";
		if (indexController.selectedNZB) {
			url = "/app/"+indexController.selectedNZB.id+"/webUpdate";
		}
		
		ServerHandler.executeOperation(url, function(statusCode, data) {
			if (statusCode==200) {
				indexController.model = data;
				callback();
			}
		});
	},

	updateNZBs : function() {
	
		var foundSelectedNZB = false;
		
		var html = "<div id=\"nzbListBackground\">";
		
		if (indexController.model && indexController.model.nzbs) {
			for (i=0; i<indexController.model.nzbs.length; i++) {
			
				//select the first asap
				if (i==0 && !indexController.selectedNZB) {
					indexController.selectedNZB = indexController.model.nzbs[0];
					indexController.initialUpdate = true;
				}
		
				var isSelected = indexController.selectedNZB&&indexController.selectedNZB.id==indexController.model.nzbs[i].id;
				var backgroundClass = (isSelected?"selected":""); 
				var progressBar = null;
				
				if (indexController.model.nzbs[i].state>20) {
					progressBar = "progressbarError";
				} else if (indexController.model.nzbs[i].isPaused) {
					progressBar = "progressbarPaused";
				} else if (indexController.model.nzbs[i].state<8) {
					progressBar = "progressbarDownloading";
				} else {
					progressBar = "progressbarProcessing";
				}
				
				var leftButtonImage = "";
				var leftButtonCommand = "";
				if (indexController.model.nzbs[i].isPaused || indexController.model.nzbs[i].state>20) {
					leftButtonImage = "images/NZBProgress_Start@2x.png";
					leftButtonCommand = "networkAPIFunctions.resumeNZB("+indexController.model.nzbs[i].id+")";
				} else if (indexController.model.nzbs[i].state!=20) {
					leftButtonImage = "images/NZBProgress_Pause@2x.png";
					leftButtonCommand = "networkAPIFunctions.pauseNZB("+indexController.model.nzbs[i].id+")";
				} else if (indexController.model.nzbs[i].state==20) {
					leftButtonImage = "images/NZBProgress_Finished@2x.png";
					leftButtonCommand = "networkAPIFunctions.cancelNZB("+indexController.model.nzbs[i].id+")";
				}
				
				var widthStyle="";
				if (indexController.model.nzbs[i].progress>0) {
					widthStyle="width: "+indexController.model.nzbs[i].progress+"%;"
				} else {
					widthStyle="display: none;";
				}
				
				html+="<div oncontextmenu=\"indexController.showContextMenu('nzbContextMenu', this); return false;\" id=\"nzb_"+indexController.model.nzbs[i].id+"\" class=\"nzbContextMenu nzbListItem "+backgroundClass+"\" onclick=\"indexController.setSelectedNZB("+indexController.model.nzbs[i].id+", this); return false;\">";
				html+="		<div class=\"progressbarBackground\"><div class=\""+progressBar+"\" style=\""+widthStyle+"\"></div></div>";
				html+="		<div class=\"nzbListItemTitle\" >"+indexController.model.nzbs[i].uiTitle+"</div>";
				html+="		<div class=\"nzbListItemState\" >"+nzbStateCodeToString(indexController.model.nzbs[i])+"</div>";
				html+="		<div class=\"nzbListItemSize\" >"+indexController.model.nzbs[i].progress.toFixed(0)+"% of "+(indexController.model.nzbs[i].totalDownloadSize/1024/1024).toFixed(1)+" MB</div>";
				html+="		<div class=\"nzbListItemLeftButton\" style=\"background-image: url('"+leftButtonImage+"');\" onclick=\""+leftButtonCommand+"; return false;\"></div>";
				
				if (navigator.userAgent.match(/(iPad|iPhone|iPod|Android)/i)) {
					html+="		<div class=\"nzbListItemRightButton\" style=\"background-image: url('images/NZBProgress_ContextMenu@2x.png');\" onclick=\"indexController.showContextMenu('nzbContextMenu', this.parentNode); return false;\"></div>";
				} else {
					html+="		<div class=\"nzbListItemRightButton\" style=\"background-image: url('images/NZBProgress_Delete@2x.png');\" onclick=\"indexController.showPanel('removeNZBPanel'); return false;\"></div>";
				}
				html+="</div>";
		
				
				if (indexController.selectedNZB && indexController.model.nzbs[i].id==indexController.selectedNZB.id) {
				
					indexController.selectedNZB = indexController.model.nzbs[i];
					if (!foundSelectedNZB) {
						foundSelectedNZB = true;
					}
				}
			}
		}
		html+="</div>";
		
		var scrollPosition = document.getElementById("nzbList").scrollTop;
		document.getElementById("nzbList").innerHTML=html;
		document.getElementById("nzbList").scrollTop = scrollPosition;
		
		if (!foundSelectedNZB) {
			indexController.selectedNZB=null;
			
			if (indexController.model) {
				indexController.model.files=null;
			}
			
			indexController.updateNZBInfoPanel();
			indexController.updateFiles();
		}
	},
	
	showContextMenu : function(menuDiv, control) {

		if (control) {
			indexController.setSelectedNZB(control.id.substring(4), control);	
		}
		
		contextMenu.showContextMenu(menuDiv, event.pageX, event.pageY, function(id) {
		
			switch(id) {
				case 'nzb_contextmenu_moveup':
					networkAPIFunctions.moveUpNZB(indexController.selectedNZB.id);
					break;
				case 'nzb_contextmenu_movedown':
					networkAPIFunctions.moveDownNZB(indexController.selectedNZB.id);
					break;
				case 'nzb_contextmenu_downloadnext':
					networkAPIFunctions.downloadNZBNext(indexController.selectedNZB.id);
					break;
				case 'nzb_contextmenu_movetop':
					networkAPIFunctions.moveTopNZB(indexController.selectedNZB.id);
					break;
				case 'nzb_contextmenu_movebottom':
					networkAPIFunctions.moveBottomNZB(indexController.selectedNZB.id);
					break;
				case 'nzb_contextmenu_pause':
					networkAPIFunctions.pauseNZB(indexController.selectedNZB.id);
					break;
				case 'nzb_contextmenu_resume':
					networkAPIFunctions.resumeNZB(indexController.selectedNZB.id);
					break;
				case 'nzb_contextmenu_remove':
					indexController.showPanel('removeNZBPanel');
					break;
			}
		}, function(id) {
		
			switch(id) {
				case 'nzb_contextmenu_moveup':
				case 'nzb_contextmenu_movetop':
					return indexController.model.nzbs.indexOf(indexController.selectedNZB)>0;
				case 'nzb_contextmenu_movedown':
				case 'nzb_contextmenu_movebottom':
					return indexController.model.nzbs.indexOf(indexController.selectedNZB)<indexController.model.nzbs.length-1;
				case 'nzb_contextmenu_downloadnext':
					return indexController.selectedNZB.state<2 && indexController.model.nzbs.indexOf(indexController.selectedNZB)>0;
				case 'nzb_contextmenu_pause':
					return indexController.selectedNZB.state<20 && !indexController.selectedNZB.isPaused;
				case 'nzb_contextmenu_resume':
					return indexController.selectedNZB.state<20 && indexController.selectedNZB.isPaused;
				default:
					return true;
			}
		});
	},
	
	updateFiles : function() {
	
		if (!indexController.selectedNZB) {
			document.getElementById("fileList").innerHTML="<div id=\"fileListBackground\"></div>";
			return;
		}

		var html="<div id=\"fileListBackground\">";
		
		if (indexController.model && indexController.model.files) {
			for (i=0; i<indexController.model.files.length; i++) {
			
				var statusLabel = fileStateCodeToString(indexController.model.files[i]);
				var color = indexController.model.files[i].state==5?"style=\"color: #CCCCCC;\"":"";
				
				if (indexController.model.files[i].state>0 && indexController.model.files[i].state<4) {
					var progress = 0;
					if (indexController.model.files[i].downloadedSize>0) {
						progress = ((indexController.model.files[i].downloadedSize/indexController.model.files[i].totalDownloadSize)*100);
					}
					statusLabel+=" ("+progress.toFixed(0)+"% of "+(indexController.model.files[i].totalDownloadSize/1024/1024).toFixed(1)+" MB)";
				} else if (indexController.model.files[i].state>=4) {
					if (indexController.model.files[i].fileSize>0) {
						statusLabel+=" ("+(indexController.model.files[i].fileSize/1024/1024).toFixed(1)+" MB)";
					}
				}
				
				html+="<div class=\"fileListItem\" "+color+">";
				html+="		<div class=\"fileListItemImage\" style=\"background-image: url('"+getImageForExtention(indexController.model.files[i].fileName)+"')\"></div>";
				html+="		<div class=\"fileListItemTitle\">"+indexController.model.files[i].fileName+"</div>";
				html+="		<div class=\"fileListItemStatus\">";
				html+= statusLabel;
				html+="		</div>";
				html+="</div>";
			}
		}
		html+="</div>";
		
		var scrollPosition = document.getElementById("fileList").scrollTop;
		document.getElementById("fileList").innerHTML=html;
		document.getElementById("fileList").scrollTop = scrollPosition;
	},
	
	updateNZBInfoPanel : function() {
		if (indexController.selectedNZB) {
			document.getElementById('nzbInfo_title').innerHTML = indexController.selectedNZB.uiTitle;
			document.getElementById('nzbInfo_state').innerHTML = nzbStateCodeToString(indexController.selectedNZB)+" ("+indexController.selectedNZB.progress.toFixed(0)+"%)";
			
			var ageDays = (Math.round(new Date().getTime()/1000.0)-indexController.selectedNZB.postDate)/60/60/24;
			document.getElementById('nzbInfo_age').innerHTML = ageDays.toFixed(0)+" days";
			
			var par2Files = 0;
			var skippedFiles = 0;
			var skippedPar2Files = 0;
			var filesCount = 0;
			
			if (indexController.model.files) {
				filesCount = indexController.model.files.length;

				for (i=0; i<indexController.model.files.length; i++) {
					var fileName = indexController.model.files[i].fileName.toLowerCase();
					
					if (fileName.match(".par2$")==".par2") {
						par2Files+=1;
						if (indexController.model.files[i].state==5) {
							skippedPar2Files+=1;
						}
					}
					
					if (indexController.model.files[i].state==5) {
						skippedFiles+=1;
					}
				}
			}
			document.getElementById('nzbInfo_repair').innerHTML = par2Files+" PAR2 ("+skippedPar2Files+" skipped)";
			document.getElementById('nzbInfo_files').innerHTML = filesCount+" files ("+skippedFiles+" skipped)";
			
		} else {
			document.getElementById('nzbInfo_title').innerHTML = 'No NZB selected';
			document.getElementById('nzbInfo_state').innerHTML = '-';
			document.getElementById('nzbInfo_repair').innerHTML = '-';
			document.getElementById('nzbInfo_files').innerHTML = '-';
			document.getElementById('nzbInfo_age').innerHTML = '-';
		}
	},
	
	updateSpeedLabel : function() {
		var speed = (indexController.model && indexController.model.speed)?indexController.model.speed:0;
		document.getElementById("toolbarSpeedLabel").innerHTML="Speed: "+(speed/1024).toFixed(0)+" KB/s";
	},
	
	updateFooterLabel : function () {
		var downloading = 0;
		var paused = 0;
		var processing = 0;
		var finished = 0;
		
		if (indexController.model && indexController.model.nzbs) {
			for (i=0; i<indexController.model.nzbs.length; i++) {
				var nzb = indexController.model.nzbs[i];
				
				if (nzb.isPaused) {
					paused++;
				} else if (nzb.state<4) {
					downloading++;
				} else if (nzb.state<20) {
					processing++;
				} else if (nzb.state==20) {
					finished++;
				}
			}
		}
		
		document.getElementById('footerLabel').innerHTML = "Downloading: "+downloading+"&nbsp;&nbsp;&nbsp;&nbsp;Processing: "+processing+"&nbsp;&nbsp;&nbsp;&nbsp;Finished: "+finished+"&nbsp;&nbsp;&nbsp;&nbsp;Paused: "+paused;
	},
	
	updateFooterIcons : function() {
		var sleepWhenDoneButton = document.getElementById('footerSleepWhenDoneButtonIcon');
		var speedLimitButton = document.getElementById('footerSpeedLimitButtonIcon');
		
		sleepWhenDoneButton.style.backgroundImage = "url("+(indexController.model && indexController.model.sleepwhendone?"images/Statusbar_SleepWhenDone_Selected@2x.png":"images/Statusbar_SleepWhenDone@2x.png")+")";
		speedLimitButton.style.backgroundImage = "url("+(indexController.model && indexController.model.speedlimit_enabled?"images/Statusbar_SpeedLimit_Selected@2x.png":"images/Statusbar_SpeedLimit@2x.png")+")";
	},
	
	
	clearUI : function() {
		indexController.model = null;
		indexController.selectedNZB = null;
		indexController.initialUpdate = false;
		indexController.updateUI();
	},
	
	updateUI : function() {
		//update NZBs
		indexController.updateNZBs();
		
		//first time, quicker refresh for files
		if (indexController.initialUpdate) {
			indexController.initialUpdate = false;
			indexController.tmrUIUpdateTick();
			return;
		}
		
		//update files
		indexController.updateFiles();
				
		//update nzb info
		indexController.updateNZBInfoPanel();
		
		//speed label
		indexController.updateSpeedLabel();
		
		//footer queue status label
		indexController.updateFooterLabel();
		
		//update footer icons
		indexController.updateFooterIcons();
	},
	
	tmrUIUpdateTick : function() {
			
		//do not update when a panel is visible
		if (indexController.displayPanel) {
			return;
		}
		
		indexController.updateModel(function() {
		
			indexController.updateUI();
		});
	},
	
	btnSpeedLimitClicked : function() {
		var speedlimitTextBox = document.getElementById('tbSpeedLimit');
		var checkbox = document.getElementById('cbxSpeedLimitEnabled');
		var minutes = document.getElementById('tbSpeedLimitTime');
		var disableSpeedLimit = document.getElementById('btnDisableSpeedLimit');
		
		btnDisableSpeedLimit.disabled=!indexController.model.speedlimit_enabled;
		speedlimitTextBox.value=indexController.model.speedlimit;
		
		if (indexController.model.speedlimit_enabled==1 && indexController.model.speedlimit_minutes>=0) {
			checkbox.checked = true;
			minutes.value=indexController.model.speedlimit_minutes;
		} else {
			checkbox.checked = false;
			minutes.value = 10;
		}
		
		indexController.showPanel('speedLimitPanel');
	},
	
	btnEnableSpeedLimitClicked : function() {
		var speedlimitTextBox = document.getElementById('tbSpeedLimit');
		var checkbox = document.getElementById('cbxSpeedLimitEnabled');
		var minutes = checkbox.checked?document.getElementById('tbSpeedLimitTime').value:null;

		networkAPIFunctions.setSpeedLimit(parseInt(speedlimitTextBox.value),minutes);
		indexController.hideAllPanels();
	},
	
	btnDisableSpeedLimitClicked : function() {
		networkAPIFunctions.disableSpeedLimit();
		indexController.hideAllPanels();
	},
	
	btnAddNZBUsingURL_clicked : function() {
		ServerHandler.executeOperation("/nzb/add?url="+encodeURIComponent(document.getElementById("tbAddNZBURL").value), function(statusCode, data) {
			if (statusCode==200) {
				document.getElementById("tbAddNZBURL").value="";
				indexController.hideAllPanels();
			} else {
				alert("Failed to add NZB via URL");
			}
		});
	},
	
	btnAPIKey_clicked : function() {
		ServerHandler.apikey=document.getElementById('tbAPIKey').value.toUpperCase();
		var storeKey = document.getElementById("cbxSaveAPIKey").checked;
		
		if (storeKey) {
			try {
				$.jStorage.set('apikey',ServerHandler.apikey);
			} 
			catch (err) {
			}
		}
		
		document.getElementById('tbAPIKey').value = "";
		indexController.login();
	},
	
	onFileUploadSubmit : function() {
		document.getElementById('upload_iframe').onload = function() {
			document.getElementById('upload_iframe').onload = null;
			var iframe = document.getElementById('upload_iframe');
			var content=(iframe.contentWindow || iframe.contentDocument);
			if (content.document) {
				content=content.document;
			}
			
			var result = null;
			try {
				result = JSON.parse(content.body.innerText);
			} 
			catch (err) {
			}
			
			if (result && result.result && result.result=="ok") {
				document.getElementById("upload1").value="";
				indexController.hideAllPanels();
			} else {
				alert('Failed to upload the file');
			}
		};
	},
}
